import { injectable } from "tsyringe";
import { IActivityState } from "../models/activityState";
import { IActivity } from "../models/config";

@injectable()
export class StateDataProvider {
  private _states: {
    [controlUnitName: string]: IActivityState | undefined;
  } = {};

  public updateState(activity: IActivity, controlUnitName: string): void {
    const oldState = this._states[controlUnitName];
    this._states[controlUnitName] = {
      currentActivity: activity,
      lastActivity: oldState?.lastActivity,
    };
  }

  public deactivateState(controlUnitName: string): void {
    const oldState = this._states[controlUnitName];
    this._states[controlUnitName] = {
      currentActivity: undefined,
      lastActivity: oldState?.currentActivity,
    };
  }

  public removeState(controlUnitName: string): void {
    this._states[controlUnitName] = undefined;
  }

  public getState(controlUnitName: string): IActivityState | undefined {
    if (!this._states[controlUnitName]) {
      return undefined;
    }
    return this._states[controlUnitName];
  }

  public get states(): {
    [controlUnitName: string]: IActivityState | undefined;
  } {
    return this._states;
  }
}
