import { inject, injectable } from "tsyringe";
import { RemoteKey } from "../accessories/controlUnit";
import { HarmonyDataProvider } from "../dataProviders/harmonyDataProvider";
import { StateDataProvider } from "../dataProviders/stateDataProvider";
import { HarmonyDevice } from "../models/harmonyDevice";

@injectable()
export class CommandService {
  constructor(
    @inject(StateDataProvider) private _stateDataProvider: StateDataProvider,
    @inject(HarmonyDataProvider)
    private _harmonyDataProvider: HarmonyDataProvider
  ) {}

  /**
   * Send key press for current activity.
   *
   * @param controlUnitName The name of the control unit to act on.
   * @param key The key to send.
   */
  public sendKeyPress = async (controlUnitName: string, key: any) => {
    let currentActivity = this._stateDataProvider.getState(controlUnitName);
    if (currentActivity) {
      let commandName: string = "";

      let device: HarmonyDevice = this._harmonyDataProvider.getDeviceByName(
        currentActivity.currentActivity!.ControlDevice
      );
      switch (key) {
        case RemoteKey.ARROW_UP: {
          commandName = "Direction Up";
          break;
        }
        case RemoteKey.ARROW_DOWN: {
          commandName = "Direction Down";
          break;
        }
        case RemoteKey.ARROW_LEFT: {
          commandName = "Direction Left";
          break;
        }
        case RemoteKey.ARROW_RIGHT: {
          commandName = "Direction Right";
          break;
        }
        case RemoteKey.SELECT: {
          commandName = "Select";
          break;
        }
        case RemoteKey.PLAY_PAUSE: {
          commandName = "Pause";
          break;
        }
        case RemoteKey.INFORMATION: {
          commandName = "Menu";
          break;
        }
        case RemoteKey.BACK: {
          commandName = "Back";
          break;
        }
        case RemoteKey.EXIT: {
          commandName = "Back";
          break;
        }
      }

      await this._harmonyDataProvider.sendCommand(commandName, device);
    }
  };
}
